@@ -32,13 +32,13 @@ from trac.config import _TRUE_VALUES
 LDAP_MODULE_CONFIG = [ 'enable', 'permfilter', 
 					   'global_perms', 'manage_groups'
 					   'cache_ttl', 'cache_size',
-					   'group_bind', 'store_bind',
-					   'user_rdn', 'group_rdn' ]
+					   'group_bind', 'perm_bind']
 
-LDAP_DIRECTORY_PARAMS = [ 'host', 'port', 'use_tls', 'base_dn',
+LDAP_DIRECTORY_PARAMS = [ 'host', 'port', 'use_tls',
+						  'base_dn', 'user_rdn', 'group_rdn', 
 						  'bind_user', 'bind_passwd',
 						  'group_class', 'group_member_attr', 'group_member_attr_is_dn',
-						  'group_rdn_attr', 'user_rdn_attr', 'perm_attr']
+						  'user_class', 'user_name_attr', 'perm_attr']
 						  
 GROUP_PREFIX = '@'
 
@@ -56,7 +56,6 @@ class LdapPermissionGroupProvider(Component):
 		self.enabled = self.config.getbool('ldap', 'enable')
 		if not self.enabled:
 			return
-		self.util = LdapUtil(self.config)
 		# LDAP connection
 		self._ldap = ldap
 		# LDAP connection config
@@ -146,10 +145,11 @@ class LdapPermissionGroupProvider(Component):
 	
 	def _get_user_groups(self, username):
 		"""Returns a list of all groups a user belongs to"""
+		user_dn = self._ldap.get_user_dn(username)
 		ldap_groups = self._ldap.get_groups()
 		groups = []
 		for group in ldap_groups:
-			if self._ldap.is_in_group(self.util.user_attrdn(username), group):
+			if self._ldap.is_in_group(user_dn, group):
 				m = DN_RE.search(group)
 				if m:
 					groupname = GROUP_PREFIX + m.group('rdn')
@@ -262,7 +262,7 @@ class LdapPermissionStore(Component):
 		"""Store the new permission for the user in the LDAP directory"""
 		if not self.enabled:
 			raise TracError("LdapPermissionStore is not enabled")
-		if self.manage_groups and self.util.is_group(action):
+		if self.manage_groups and self._is_group(action):
 			self._flush_group_cache(username)
 			self._add_user_to_group(username.encode('ascii'), action)
 			return
@@ -273,7 +273,7 @@ class LdapPermissionStore(Component):
 			if action not in permlist:
 				xaction = self._build_action(action)
 				self._ldap.add_attribute(uid, self._ldap.permattr, xaction)
-			if self.util.is_group(username):
+			if self._is_group(username):
 				# flush the cache as group dependencies are not known 
 				self.flush_cache()
 			else:
@@ -286,7 +286,7 @@ class LdapPermissionStore(Component):
 		"""Remove the permission for the user from the LDAP directory"""
 		if not self.enabled:
 			raise TracError("LdapPermissionStore is not enabled")
-		if self.manage_groups and self.util.is_group(action):
+		if self.manage_groups and self._is_group(action):
 			self._flush_group_cache(username)
 			self._remove_user_from_group(username.encode('ascii'), action)
 			return
@@ -297,7 +297,7 @@ class LdapPermissionStore(Component):
 				action = action.encode('ascii')
 				xaction = self._build_action(action)
 				self._ldap.delete_attribute(uid, self._ldap.permattr, xaction)
-				if self.util.is_group(username):
+				if self._is_group(username):
 					# flush the cache as group dependencies are not known 
 					self.flush_cache()
 				else:
@@ -448,62 +448,9 @@ class LdapPermissionStore(Component):
 				if isinstance(provider, LdapPermissionGroupProvider):
 					provider.flush_cache(username)
 	
-
-class LdapUtil(object):
-	"""Utilities for LDAP data management"""
-	
-	def __init__(self, config):
-		for k, default in [('group_rdn_attr', 'cn'), 
-						   ('user_rdn_attr', 'uid'),
-						   ('base_dn', None),
-						   ('user_rdn', None),
-						   ('group_rdn', None)]:
-			
-			v = config.get('ldap', k, default)
-			if v: v = v.encode('ascii').lower()
-			self.__setattr__(k, v)
-	
-	def is_group(self, username):
+	def _is_group(self, username):
 		return username.startswith(GROUP_PREFIX)
 	
-	def create_dn(self, username):
-		"""Create a user or group LDAP DN from his/its name"""
-		if username.startswith(GROUP_PREFIX):
-			return self.create_dn_for_group(username[len(GROUP_PREFIX):])
-		else:
-			return self.create_dn_for_user(username)
-	
-	def create_dn_for_group(self, group):
-		"""Build the dn for a group"""
-		if self.group_rdn:
-			return "%s=%s,%s,%s" % (self.group_rdn_attr, group, self.group_rdn, self.base_dn)
-		else:
-			return "%s=%s,%s" % (self.group_rdn_attr, group, self.base_dn)
-	
-	def create_dn_for_user(self, user):
-		"""Build the dn for a user"""
-		if self.user_rdn:
-			return "%s=%s,%s,%s" % (self.user_rdn_attr, user, self.user_rdn, self.base_dn)
-		else:
-			return "%s=%s,%s" % (self.user_rdn_attr, user, self.base_dn)
-	
-	def extract_user_from_dn(self, dn):
-		m = DN_RE.search(dn)
-		if m:
-			sub = m.group('base').lower()
-			base_dn_len = len(self.base_dn)
-			if sub[len(sub)-base_dn_len:].lower() != self.bas_edn:
-				return None
-			
-			rdn = sub[:-base_dn_len-1]
-			if rdn == self.group_rdn:
-				if m.group('attr').lower() == self.group_rdn_attr:
-					return GROUP_PREFIX + m.group('rdn')
-			elif rdn == self.user_rdn:
-				if m.group('attr').lower() == self.user_rdn_attr:
-					return m.group('rdn')
-		return None
-	
 
 class LdapConnection(object):
 	"""
@@ -551,9 +498,14 @@ class LdapConnection(object):
 		self._ds.unbind_s()
 		self._ds = None
 	
+	# Group management
 	def get_groups(self):
-		"""Return a list of available group dns"""
-		groups = self.get_dn(self.base_dn, '(objectclass=%s)' % self.group_class)
+		"""Return a list of available group DNs"""
+		if self.group_rdn:
+			group_base_dn = "%s,%s" % (self.group_rdn, self.base_dn)
+		else:
+			group_base_dn = self.base_dn
+		groups = self.get_dn(group_base_dn, '(objectclass=%s)' % self.group_class)
 		return groups
 	
 	def is_in_group(self, user_dn, group_dn):
@@ -561,11 +513,7 @@ class LdapConnection(object):
 		if self.group_member_attr_is_dn:
 			username = user_dn 
 		else:
-			m = re.match('[^=]+=([^,]+)', user_dn)
-			if m is None:
-				self.log.warn('Malformed userdn: %s' % user_dn)
-				return False
-			username = m.group(1)
+			username = self.get_user_name(user_dn)
 		
 		for attempt in range(2):
 			cr = self._compare(group_dn, self.group_member_attr, username)
@@ -573,6 +521,40 @@ class LdapConnection(object):
 				return cr
 		return False
 	
+	# User management
+	def get_users(self):
+		"""Retuns a list of available user DNs"""
+		if self.user_rdn:
+			user_base_dn = "%s,%s" % (self.user_rdn, self.base_dn)
+		else:
+			user_base_dn = self.base_dn
+		
+		users = self.get_dn(user_base_dn, '(objectClass=%s)' % self.user_class)
+		return users
+	
+	def get_user_dn(self, username):
+		"""Returns the DN for the specefied username"""
+		if self.user_rdn:
+			user_base_dn = "%s,%s" % (self.user_rdn, self.base_dn)
+		else:
+			user_base_dn = self.base_dn
+		
+		user_dns = self.get_dn(user_base_dn, "(&(objectClass=%s)(%s=%s))" % (self.user_class, self.user_name_attr, username))
+		if len(user_dns) == 1:
+			return user_dns[0]
+		else:
+			return None
+	
+	def get_user_name(self, dn):
+		"""Returns the username for the specefied DN"""
+		usernames = get_attribute(dn, self.user_name_attr)
+		
+		if len(usernames) == 1:
+			return username[0]
+		else:
+			return None
+	
+	# LDAP helper
 	def get_dn(self, base_dn, filterstr):
 		"""Return a list of dns that satisfy the LDAP filter"""
 		dns = []
